/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <POURRIER Thibaud> wrote this file. As long as you retain this
 * notice you can do whatever you want with this stuff. If we meet some day,
 * and you think this stuff is worth it, you can buy me a beer in return.
 * Poul-Henning Kamp.
 * ----------------------------------------------------------------------------
 */

/******************************************************************************
		"main.c"
		This file is the main file of the project BOBY
		CPU  = ATmega48
		FCPU = 8MHz.
*******************************************************************************/

#include "main.h"

//****************************************************** Main
int main()
{
	Init();
	uchar i = 0;
#ifdef DEBUG
	Hello_led();
#endif
	
	if(ADCH_min == 255)
		Calibration();
	else
	{
		Write_str("Presser OK pour calibration", 0x00);
		dms(1000);
		if(!OK)
			Calibration();
	}

	Enter_target();
	Motor_on();

// Infos display
	Write_char('|', 0x0A);
	Write_char('|', 0x4A);
	Write_str("Trs voulus", 0x00);
	Write_str("Trs Reels", 0x40);
	Write_str("Vit-", 0x0C);
	Write_str("esse", 0x4C);
	dms(3000);

// Datas display
	Clear_screen();
	Write_char('|', 0x0A);
	Write_char('|', 0x4A);
	Write_str("V:", 0x00);
	Write_str("R:", 0x40);
	Write_str("V:", 0x0C);
	for(i=0 ; i<DIGIT ; i++)
		Write_nbr(Target[i], 0x02+i, 42);

	while(!Compare())
	{
		if(ADCH < ADCH_min+1)
		{
			OCR0B = 0;
			Motor_off();
		}
		else
		{
			TCCR0A |= (1<<COM0B1);
			OCR0B = ADCH;
		}

		for(i=0 ; i<DIGIT ; i++)
			Write_nbr(Current[i], 0x42+i, 42);

		Write_nbr((OCR0B*100)/ADCH_max, 0x4C, '%');
	}
	
	Motor_off();	// Current == target, stop motor

	Clear_screen();
	Write_str("Elle est finite ta bobine!", 0x00);
	for(i=0 ; i<DIGIT ; i++)
		Write_nbr(Current[i], 0x50-DIGIT+i, 42);
}


//****************************************************** Init
void Init()
{
	uchar i = 0;

	cli();	// Interrupts disable

	Init_screen();
	Write_str("Init...", 0x00);

// I/O
	 DDRC |= (1<<PC4);		// PB1 out (LED)

	PORTB |= (1<<PB6)|(1<<PB7);	// PB0,6,7 in (+/<), PullUp enabled
	PORTD |= (1<<PD1)|(1<<PD0)|(1<<PD4);				// PD4,7 in (-/OK/>), PullUp enabled

	 DDRD |= (1<<PD5);		// PD5 out (motor)

//ADC
	 ADMUX |= (0<<REFS1)|(1<<REFS0) | (1<<ADLAR);		// Vref = AVcc | Left adjusted |
	 ADMUX |= (0<<MUX3)|(1<<MUX2)|(0<<MUX1)|(1<<MUX0);	// Channel 5 selected
	ADCSRA |= (1<<ADEN) | (1<<ADSC) | (1<<ADATE);		// ADC enabled | ADC started | Auto trigger mode enabled |
	ADCSRA |= (1<<ADPS2)|(1<<ADPS1)|(0<<ADPS0);			// ADC prescaler = 64 -> ADC_CLK = 125KHz @8MHz

// Motor PWM (on timer 0)
	TCCR0A |= (0<<COM0B1)|(0<<COM0B0) | (1<<WGM01)|(1<<WGM00);
	TCCR0B |= (0<<WGM02) | (0<<CS02)|(0<<CS01)|(1<<CS00);	// Fast PWM mode selected | Div = 64

// IR switches
	EICRA |= (1<<ISC01)|(0<<ISC00);	// Interrupt on falling edge
	EIMSK |= (1<<INT0);				// Interrupt enabled

// Vars
	for(i=0 ; i<DIGIT ; i++)
	{
		Target[i] = 0;
		Current[i] = 0;
	}
	ADCH_min = eeprom_read_byte((const uchar*)0);
	ADCH_max = eeprom_read_byte((const uchar*)1);

	sei();	// Enable interrupts = SREG |= _BV(7)
}


//****************************************************** Calibration
void Calibration()
{
	ADCH_min = ADCH_max = 42;

// Notice display
	Write_str("Mettre la vites-se au max, puis", 0x00);
	dms(3000);
	Clear_screen();
	Write_str("au min, et faireOK pour valider", 0x00);
	dms(3000);
	Clear_screen();
	Write_str("Courant:", 0x00);
	Write_str("Min:    Max:", 0x40);

	while(OK)
	{
		if(ADCH < ADCH_min) ADCH_min = ADCH;
		if(ADCH > ADCH_max) ADCH_max = ADCH;
		Write_nbr(ADCH, 0x08, 0);
		Write_nbr(ADCH_min, 0x44, 0);
		Write_nbr(ADCH_max, 0x4C, 0);
		dms(100);
	}

	eeprom_write_byte((uchar*)0, (uchar)ADCH_min);
	eeprom_write_byte((uchar*)1, (uchar)ADCH_max);

	Clear_screen();
	Write_str("Calibration OK!", 0x00);
	dms(2000);
}


//****************************************************** Enter target
void Enter_target()
{
	uchar i = 0, pos = 0;	// pos: cursor position

// Display
	CURSOR_ON;
	Clear_screen();
	Write_str("Tours voulus?", 0x00);
	Write_cmd(128+TARGET_ADD);

	for(i=0 ; i<DIGIT ; i++)
		Write_nbr(Target[i], TARGET_ADD+i, 42);

	while(OK)	// While OK button is not pressed
	{
		if(!PLUS)		// + pressed
		{
			if(Target[pos] >= 9)
				Target[pos] = 0;
			else
				Target[pos]++;

			Write_nbr(Target[pos], TARGET_ADD+pos, 42);
			LED_ON;
		}
		else if(!MINUS)	// - pressed
		{
			if(Target[pos] <= 0)
				Target[pos] = 9;
			else
				Target[pos]--;

			Write_nbr(Target[pos], TARGET_ADD+pos, 42);
			LED_ON;
		}
		else if(!RIGHT)	// > pressed
		{
			if(pos == (DIGIT-1))
				pos = 0;
			else
				pos++;
			LED_ON;
		}
		else if(!LEFT)	// < pressed
		{
			if(pos == 0)
				pos = (DIGIT-1);
			else
				pos--;
			LED_ON;
		}

		Write_cmd(128+TARGET_ADD+pos);	// Replace cursor
		dms(75);	// anti rebond
		LED_OFF;
		dms(75);	// anti rebond
	}
	
	CURSOR_OFF;	// Display
}


//****************************************************** Motor control
void Motor_on()
{
	Clear_screen();
	LED_ON;
	while(ADCH > ADCH_min+1)	// The motor can't start if V != 0 (for slow start)
		Write_str("Attention!      Vitesse > 0!", 0x00);
	LED_OFF;
	Clear_screen();
	Write_str("Vitesse OK", 0x00);
	dms(500);
	Clear_screen();

	TCCR0A |= (1<<COM0B1);
}

void Motor_off()
{
	TCCR0A &= ~(1<<COM0B1);
	PORTD &= ~_BV(PD6);
}


//******************************************************* IR switch 0 interrupt
ISR(INT0_vect)
{
	if(IRSW1)
		Count();
	else
		Uncount();
}


//****************************************************** Spin counter
void Count()
{
	uchar i = 0;

	Current[DIGIT-1]++;

	for(i=DIGIT-1 ; i>=1 ; i--)
	{
		if(Current[i] > 9)
		{
			Current[i] = 0;
			Current[i-1]++;
		}
	}
}

void Uncount()
{
	uchar i = 0;

	Current[DIGIT-1]--;

	for(i=DIGIT-1 ; i>=1 ; i--)
	{
		if(Current[i] == 255)	// =-1
		{
			Current[i] = 9;
			Current[i-1]--;
		}
	}
}


//****************************************************** Compare Current and Target
uchar Compare()
{
	uchar i = 0;

	for(i=0 ; i<DIGIT ; i++)
		if(Current[i] != Target[i])
			return 0;

	return 42;	// else return 42
}


//****************************************************** DEBUG
#ifdef DEBUG

void Hello_led()
{
	uchar i = 0;
	
	for(i=0 ; i<3; i++)
	{
		LED_ON;
		dms(70);
		LED_OFF;
		dms(70);
	}
}

#endif
