/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <POURRIER Thibaud> wrote this file. As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return.
 * Poul-Henning Kamp.
 * ----------------------------------------------------------------------------
 */

/******************************************************************************
		"screen4b.c"
		This file contain all functions to control a LCD screen in 4 bits mode.
		See also "screen4b.h".
		
>>>>	IMPORTANT: Maybe you should change pin assignation!
		
		Note: 	dms(); and dus(); were redefined in "main.h" are respectively
				_delay-ms(); and _delay_us();.
*******************************************************************************/


#include "main.h"

void Init_screen()
{
// I/O
	DDRB |= (1<<PB1)|(1<<PB2);	// PB1,2 out: EN/RS
	DDRC |= 0x0F;	// PC0..3 out: DATA

	dms(30);	// Wait screen power on

// Init commands:
	Write_cmd(0b00100010);	// Write 0b0010 several times (RTFM for more details)
	Write_cmd(0b00101000);	// Function set : 4 bits mode (third time), 2 lines, 5*8px
	Write_cmd(0b00001100);	// Display Control : display ON, Cursor OFF, blink OFF
	Clear_screen();
	Write_cmd(0b00000110);	// Entry Mode : cursor move to right, entire shift OFF
	Return_home();
}

void Write_cmd(unsigned char byte)	// Write byte
{
	RS_OFF;
	dus(1);

	CLEAR_DATA;
	DATA |= byte>>4;	// MSBs

	EN_ON;
	EN_OFF;	// Write

	CLEAR_DATA;
	DATA |= byte & 0x0F;	// LSBs

	EN_ON;
	EN_OFF;	// Write

	dus(100);
}

void Write_char(unsigned char data, unsigned char add)	// Write caracter on screen
{
	// set DDRAM add
	Write_cmd(128 + add);
	
	// set DDRAM caracter
	RS_ON;
	dus(1);

	CLEAR_DATA;
	DATA |= data>>4;	// MSBs

	EN_ON;
	EN_OFF;	// Write

	CLEAR_DATA;
	DATA |= data & 0x0F;	// LSBs

	EN_ON;
	EN_OFF;	// Write

	dus(100);
}

#ifdef SCR_2x8
	void Write_str(char str[16], unsigned char add)
#else
	void Write_str(char str[32], unsigned char add)
#endif
{
	unsigned char i;

	for(i=0 ; i<strlen(str) ; i++)
	{
		Write_char(str[i], add);
		add++;
		#ifdef SCR_2x8
			if(add == 0x08) add = 0x40;
		#else
			if(add == 0x10) add = 0x40;
		#endif
	}
}

void Write_nbr(int nbr, unsigned char add, uchar mode)	// Write number
{
	char buffer[10];
	
	if(mode == 42)
		Write_str(itoa(nbr, buffer, 10), add);
	else if(mode == '%')
		Write_str(strcat(itoa(nbr, buffer, 10), "% "), add);
	else
		Write_str(strcat(itoa(nbr, buffer, 10), " "), add);
}

void Clear_screen()
{
	Write_cmd(1);
	dms(2);
}

void Return_home()
{
	Write_cmd(2);
	dms(2);
}
